@@ -1,7 +1,7 @@
 #!/usr/bin/python2.5
 
 """
-DialCentral - Front end for Google's Grand Central service.
+DialCentral - Front end for Google's GoogleVoice service.
 Copyright (C) 2008  Mark Bergman bergman AT merctech DOT com
 
 This library is free software; you can redistribute it and/or
@@ -17,6 +17,8 @@ Lesser General Public License for more details.
 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+
+@bug When switching to tab while logging in, it doesn't refresh once logged int
 """
 
 
@@ -117,36 +119,38 @@ class Dialcentral(object):
 			'recent_scrolledwindow',
 			'message_scrolledwindow',
 			'contacts_scrolledwindow',
-			"phoneSelectionMessage_scrolledwindow",
+			"phoneSelectionMessages_scrolledwindow",
+			"smsMessages_scrolledwindow",
+		):
+			hildonize.hildonize_scrollwindow(self._widgetTree.get_widget(scrollingWidget))
+		for scrollingWidget in (
 			"phonetypes_scrolledwindow",
-			"smsMessage_scrolledwindow",
 			"smsMessage_scrolledEntry",
 		):
-			hildonize.set_thumb_scrollbar(self._widgetTree.get_widget(scrollingWidget))
+			hildonize.hildonize_scrollwindow_with_viewport(self._widgetTree.get_widget(scrollingWidget))
 
-		hildonize.hildonize_menu(self._window, self._widgetTree.get_widget("dialpad_menubar"))
+		replacementButtons = [gtk.Button("Test")]
+		menu = hildonize.hildonize_menu(
+			self._window,
+			self._widgetTree.get_widget("dialpad_menubar"),
+			replacementButtons
+		)
 
-		if hildonize.IS_HILDON:
+		if hildonize.IS_HILDON_SUPPORTED:
 			self._window.connect("key-press-event", self._on_key_press)
 			self._window.connect("window-state-event", self._on_window_state_change)
 		else:
 			logging.warning("No hildonization support")
 
-
 		hildonize.set_application_title(self._window, "%s" % constants.__pretty_app_name__)
 
-		callbackMapping = {
-			"on_dialpad_quit": self._on_close,
-		}
-		self._widgetTree.signal_autoconnect(callbackMapping)
-
 		self._window.connect("destroy", self._on_close)
 		self._window.set_default_size(800, 300)
 		self._window.show_all()
 
 		self._loginSink = gtk_toolbox.threaded_stage(
 			gtk_toolbox.comap(
-				self.attempt_login,
+				self._attempt_login,
 				gtk_toolbox.null_sink(),
 			)
 		)
@@ -205,9 +209,15 @@ class Dialcentral(object):
 					self._errorDisplay.push_exception()
 				alarm_handler = None
 				logging.warning("No notification support")
-			if hildonize.IS_HILDON:
-				import led_handler
-				self._ledHandler = led_handler.LedHandler()
+			if hildonize.IS_HILDON_SUPPORTED:
+				try:
+					import led_handler
+					self._ledHandler = led_handler.LedHandler()
+				except Exception, e:
+					logging.exception('LED Handling failed: "%s"' % str(e))
+					self._ledHandler = None
+			else:
+				self._ledHandler = None
 
 			try:
 				import conic
@@ -287,12 +297,13 @@ class Dialcentral(object):
 			callbackMapping = {
 				"on_paste": self._on_paste,
 				"on_refresh": self._on_menu_refresh,
-				"on_rotate": self._on_menu_rotate,
 				"on_clearcookies_clicked": self._on_clearcookies_clicked,
-				"on_notebook_switch_page": self._on_notebook_switch_page,
 				"on_about_activate": self._on_about_activate,
 			}
-			self._widgetTree.signal_autoconnect(callbackMapping)
+			if hildonize.GTK_MENU_USED:
+				self._widgetTree.signal_autoconnect(callbackMapping)
+			self._notebook.connect("switch-page", self._on_notebook_switch_page)
+			self._widgetTree.get_widget("clearcookies").connect("clicked", self._on_clearcookies_clicked)
 
 			with gtk_toolbox.gtk_lock():
 				self._originalCurrentLabels = [
@@ -318,10 +329,11 @@ class Dialcentral(object):
 		finally:
 			self._spawn_attempt_login(2)
 
-	def attempt_login(self, numOfAttempts = 10, force = False):
-		"""
-		@todo Handle user notification better like attempting to login and failed login
+	def _spawn_attempt_login(self, *args):
+		self._loginSink.send(args)
 
+	def _attempt_login(self, numOfAttempts = 10, force = False):
+		"""
 		@note This must be run outside of the UI lock
 		"""
 		try:
@@ -336,20 +348,19 @@ class Dialcentral(object):
 					serviceId = self._defaultBackendId
 					loggedIn = True
 				except Exception, e:
-					logging.exception('Session refresh failed with the following message "%s"' % e.message)
+					logging.exception('Session refresh failed with the following message "%s"' % str(e))
 
 			if not loggedIn:
 				loggedIn, serviceId = self._login_by_user(numOfAttempts)
 
 			with gtk_toolbox.gtk_lock():
 				self._change_loggedin_status(serviceId)
+				if loggedIn:
+					hildonize.show_information_banner(self._window, "Logged In")
 		except Exception, e:
 			with gtk_toolbox.gtk_lock():
 				self._errorDisplay.push_exception()
 
-	def _spawn_attempt_login(self, *args):
-		self._loginSink.send(args)
-
 	def refresh_session(self):
 		"""
 		@note Thread agnostic
@@ -398,6 +409,9 @@ class Dialcentral(object):
 				credentials = self._credentialsDialog.request_credentials(
 					defaultCredentials = self._credentials
 				)
+				if not self._phoneBackends[tmpServiceId].get_callback_number():
+					# subtle reminder to the users to configure things
+					self._notebook.set_current_page(self.ACCOUNT_TAB)
 			username, password = credentials
 			loggedIn = self._phoneBackends[tmpServiceId].login(username, password)
 
@@ -407,6 +421,7 @@ class Dialcentral(object):
 			logging.info("Logged into %r through user request" % self._phoneBackends[serviceId])
 		else:
 			serviceId = self.NULL_BACKEND
+			self._notebook.set_current_page(self.ACCOUNT_TAB)
 
 		return loggedIn, serviceId
 
@@ -442,6 +457,7 @@ class Dialcentral(object):
 		if self._phoneBackends[self._selectedBackendId].get_callback_number() is None:
 			self._phoneBackends[self._selectedBackendId].set_sane_callback()
 		self._accountViews[self._selectedBackendId].update()
+		self._refresh_active_tab()
 
 		self._selectedBackendId = newStatus
 
@@ -503,7 +519,6 @@ class Dialcentral(object):
 					),
 				)
 
-		# @todo down here till this issue is fixed
 		try:
 			previousOrientation = config.getint(constants.__pretty_app_name__, "orientation")
 			if previousOrientation == gtk.ORIENTATION_HORIZONTAL:
@@ -722,13 +737,14 @@ class Dialcentral(object):
 
 			if not loggedIn:
 				self._errorDisplay.push_message(
-					"Backend link with grandcentral is not working, please try again"
+					"Backend link with GoogleVoice is not working, please try again"
 				)
 				return
 
 			dialed = False
 			try:
 				self._phoneBackends[self._selectedBackendId].send_sms(number, message)
+				hildonize.show_information_banner(self._window, "Sending to %s" % number)
 				dialed = True
 			except Exception, e:
 				self._errorDisplay.push_exception()
@@ -750,7 +766,7 @@ class Dialcentral(object):
 
 			if not loggedIn:
 				self._errorDisplay.push_message(
-					"Backend link with grandcentral is not working, please try again"
+					"Backend link with GoogleVoice is not working, please try again"
 				)
 				return
 
@@ -758,6 +774,7 @@ class Dialcentral(object):
 			try:
 				assert self._phoneBackends[self._selectedBackendId].get_callback_number() != "", "No callback number specified"
 				self._phoneBackends[self._selectedBackendId].dial(number)
+				hildonize.show_information_banner(self._window, "Calling %s" % number)
 				dialed = True
 			except Exception, e:
 				self._errorDisplay.push_exception()
@@ -773,20 +790,11 @@ class Dialcentral(object):
 		except Exception, e:
 			self._errorDisplay.push_exception()
 
-	def _on_menu_rotate(self, *args):
-		try:
-			orientation = gtk_toolbox.get_screen_orientation()
-			if orientation == gtk.ORIENTATION_HORIZONTAL:
-				hildonize.window_to_portrait(self._window)
-			elif orientation == gtk.ORIENTATION_VERTICAL:
-				hildonize.window_to_landscape(self._window)
-		except Exception, e:
-			self._errorDisplay.push_exception()
-
 	def _on_paste(self, *args):
 		try:
 			contents = self._clipboard.wait_for_text()
-			self._dialpads[self._selectedBackendId].set_number(contents)
+			if contents is not None:
+				self._dialpads[self._selectedBackendId].set_number(contents)
 		except Exception, e:
 			self._errorDisplay.push_exception()
 
@@ -796,7 +804,7 @@ class Dialcentral(object):
 			dlg.set_name(constants.__pretty_app_name__)
 			dlg.set_version("%s-%d" % (constants.__version__, constants.__build__))
 			dlg.set_copyright("Copyright 2008 - LGPL")
-			dlg.set_comments("Dialcentral is a touch screen enhanced interface to your GoogleVoice/Grandcentral account.  This application is not affiliated with Google in any way")
+			dlg.set_comments("Dialcentral is a touch screen enhanced interface to your GoogleVoice account.  This application is not affiliated with Google in any way")
 			dlg.set_website("http://gc-dialer.garage.maemo.org/")
 			dlg.set_authors(["<z2n@merctech.com>", "Eric Warnke <ericew@gmail.com>", "Ed Page <edpage@byu.net>"])
 			dlg.run()
@@ -821,7 +829,7 @@ def run_dialpad():
 	#with gtk_toolbox.flock(_lock_file, 0):
 	gtk.gdk.threads_init()
 
-	if hildonize.IS_HILDON:
+	if hildonize.IS_HILDON_SUPPORTED:
 		gtk.set_application_name(constants.__pretty_app_name__)
 	handle = Dialcentral()
 	gtk.main()
