@@ -1,7 +1,7 @@
 #!/usr/bin/python2.5
 
 """
-DialCentral - Front end for Google's Grand Central service.
+DialCentral - Front end for Google's GoogleVoice service.
 Copyright (C) 2008  Mark Bergman bergman AT merctech DOT com
 
 This library is free software; you can redistribute it and/or
@@ -17,6 +17,9 @@ Lesser General Public License for more details.
 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+
+@todo Touch selector for callback number
+@todo Alternate UI for dialogs (stackables)
 """
 
 from __future__ import with_statement
@@ -284,23 +287,24 @@ class PhoneTypeSelector(object):
 		self._cancelButton = self._widgetTree.get_widget("cancel_button")
 		self._cancelButton.connect("clicked", self._on_phonetype_cancel)
 
+		self._messagemodel = gtk.ListStore(gobject.TYPE_STRING)
+		self._messagesView = self._widgetTree.get_widget("phoneSelectionMessages")
+		self._scrollWindow = self._messagesView.get_parent()
+
 		self._typemodel = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
 		self._typeviewselection = None
-
-		self._message = self._widgetTree.get_widget("phoneSelectionMessage")
-		self._messageViewport = self._widgetTree.get_widget("phoneSelectionMessage_viewport")
-		self._scrollWindow = self._widgetTree.get_widget("phoneSelectionMessage_scrolledwindow")
 		self._typeview = self._widgetTree.get_widget("phonetypes")
 		self._typeview.connect("row-activated", self._on_phonetype_select)
 
 		self._action = self.ACTION_CANCEL
 
-	def run(self, contactDetails, message = "", parent = None):
+	def run(self, contactDetails, messages = (), parent = None):
 		self._action = self.ACTION_CANCEL
+
+		# Add the column to the phone selection tree view
 		self._typemodel.clear()
 		self._typeview.set_model(self._typemodel)
 
-		# Add the column to the treeview
 		textrenderer = gtk.CellRendererText()
 		numberColumn = gtk.TreeViewColumn("Phone Numbers", textrenderer, text=0)
 		self._typeview.append_column(numberColumn)
@@ -309,32 +313,49 @@ class PhoneTypeSelector(object):
 		typeColumn = gtk.TreeViewColumn("Phone Type", textrenderer, text=1)
 		self._typeview.append_column(typeColumn)
 
-		self._typeviewselection = self._typeview.get_selection()
-		self._typeviewselection.set_mode(gtk.SELECTION_SINGLE)
-
 		for phoneType, phoneNumber in contactDetails:
 			display = " - ".join((phoneNumber, phoneType))
 			display = phoneType
 			row = (phoneNumber, display)
 			self._typemodel.append(row)
 
+		self._typeviewselection = self._typeview.get_selection()
+		self._typeviewselection.set_mode(gtk.SELECTION_SINGLE)
 		self._typeviewselection.select_iter(self._typemodel.get_iter_first())
-		if message:
-			self._message.set_markup(message)
-			self._message.show()
+
+		# Add the column to the messages tree view
+		self._messagemodel.clear()
+		self._messagesView.set_model(self._messagemodel)
+
+		textrenderer = gtk.CellRendererText()
+		textrenderer.set_property("wrap-mode", pango.WRAP_WORD)
+		textrenderer.set_property("wrap-width", 450)
+		messageColumn = gtk.TreeViewColumn("")
+		messageColumn.pack_start(textrenderer, expand=True)
+		messageColumn.add_attribute(textrenderer, "markup", 0)
+		messageColumn.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
+		self._messagesView.append_column(messageColumn)
+		self._messagesView.set_headers_visible(False)
+
+		if messages:
+			for message in messages:
+				row = (message, )
+				self._messagemodel.append(row)
+			self._messagesView.show()
+			self._scrollWindow.show()
+			messagesSelection = self._messagesView.get_selection()
+			messagesSelection.select_path((len(messages)-1, ))
 		else:
-			self._message.set_markup("")
-			self._message.hide()
+			self._messagesView.hide()
+			self._scrollWindow.hide()
 
 		if parent is not None:
 			self._dialog.set_transient_for(parent)
 
 		try:
 			self._dialog.show()
-			adjustment = self._scrollWindow.get_vadjustment()
-			dx = self._message.get_allocation().height - self._messageViewport.get_allocation().height
-			dx = max(dx, 0)
-			adjustment.value = dx
+			if messages:
+				self._messagesView.scroll_to_cell((len(messages)-1, ))
 
 			userResponse = self._dialog.run()
 		finally:
@@ -349,13 +370,16 @@ class PhoneTypeSelector(object):
 			self._action = self.ACTION_CANCEL
 
 		if self._action == self.ACTION_SEND_SMS:
-			smsMessage = self._smsDialog.run(phoneNumber, message, parent)
+			smsMessage = self._smsDialog.run(phoneNumber, messages, parent)
 			if not smsMessage:
 				phoneNumber = ""
 				self._action = self.ACTION_CANCEL
 		else:
 			smsMessage = ""
 
+		self._messagesView.remove_column(messageColumn)
+		self._messagesView.set_model(None)
+
 		self._typeviewselection.unselect_all()
 		self._typeview.remove_column(numberColumn)
 		self._typeview.remove_column(typeColumn)
@@ -389,7 +413,6 @@ class PhoneTypeSelector(object):
 
 
 class SmsEntryDialog(object):
-
 	"""
 	@todo Add multi-SMS messages like GoogleVoice
 	"""
@@ -407,19 +430,41 @@ class SmsEntryDialog(object):
 		self._cancelButton.connect("clicked", self._on_cancel)
 
 		self._letterCountLabel = self._widgetTree.get_widget("smsLetterCount")
-		self._message = self._widgetTree.get_widget("smsMessage")
-		self._messageViewport = self._widgetTree.get_widget("smsMessage_viewport")
-		self._scrollWindow = self._widgetTree.get_widget("smsMessage_scrolledwindow")
+
+		self._messagemodel = gtk.ListStore(gobject.TYPE_STRING)
+		self._messagesView = self._widgetTree.get_widget("smsMessages")
+		self._scrollWindow = self._messagesView.get_parent()
+
 		self._smsEntry = self._widgetTree.get_widget("smsEntry")
 		self._smsEntry.get_buffer().connect("changed", self._on_entry_changed)
 
-	def run(self, number, message = "", parent = None):
-		if message:
-			self._message.set_markup(message)
-			self._message.show()
+	def run(self, number, messages = (), parent = None):
+		# Add the column to the messages tree view
+		self._messagemodel.clear()
+		self._messagesView.set_model(self._messagemodel)
+
+		textrenderer = gtk.CellRendererText()
+		textrenderer.set_property("wrap-mode", pango.WRAP_WORD)
+		textrenderer.set_property("wrap-width", 450)
+		messageColumn = gtk.TreeViewColumn("")
+		messageColumn.pack_start(textrenderer, expand=True)
+		messageColumn.add_attribute(textrenderer, "markup", 0)
+		messageColumn.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
+		self._messagesView.append_column(messageColumn)
+		self._messagesView.set_headers_visible(False)
+
+		if messages:
+			for message in messages:
+				row = (message, )
+				self._messagemodel.append(row)
+			self._messagesView.show()
+			self._scrollWindow.show()
+			messagesSelection = self._messagesView.get_selection()
+			messagesSelection.select_path((len(messages)-1, ))
 		else:
-			self._message.set_markup("")
-			self._message.hide()
+			self._messagesView.hide()
+			self._scrollWindow.hide()
+
 		self._smsEntry.get_buffer().set_text("")
 		self._update_letter_count()
 
@@ -428,10 +473,9 @@ class SmsEntryDialog(object):
 
 		try:
 			self._dialog.show()
-			adjustment = self._scrollWindow.get_vadjustment()
-			dx = self._message.get_allocation().height - self._messageViewport.get_allocation().height
-			dx = max(dx, 0)
-			adjustment.value = dx
+			if messages:
+				self._messagesView.scroll_to_cell((len(messages)-1, ))
+			self._smsEntry.grab_focus()
 
 			userResponse = self._dialog.run()
 		finally:
@@ -444,6 +488,9 @@ class SmsEntryDialog(object):
 		else:
 			enteredMessage = ""
 
+		self._messagesView.remove_column(messageColumn)
+		self._messagesView.set_model(None)
+
 		return enteredMessage.strip()
 
 	def _update_letter_count(self, *args):
@@ -468,22 +515,23 @@ class SmsEntryDialog(object):
 class Dialpad(object):
 
 	def __init__(self, widgetTree, errorDisplay):
+		self._clipboard = gtk.clipboard_get()
 		self._errorDisplay = errorDisplay
 		self._smsDialog = SmsEntryDialog(widgetTree)
 
 		self._numberdisplay = widgetTree.get_widget("numberdisplay")
+		self._smsButton = widgetTree.get_widget("sms")
 		self._dialButton = widgetTree.get_widget("dial")
 		self._backButton = widgetTree.get_widget("back")
 		self._phonenumber = ""
 		self._prettynumber = ""
 
 		callbackMapping = {
-			"on_dial_clicked": self._on_dial_clicked,
-			"on_sms_clicked": self._on_sms_clicked,
 			"on_digit_clicked": self._on_digit_clicked,
-			"on_clear_number": self._on_clear_number,
 		}
 		widgetTree.signal_autoconnect(callbackMapping)
+		self._dialButton.connect("clicked", self._on_dial_clicked)
+		self._smsButton.connect("clicked", self._on_sms_clicked)
 
 		self._originalLabel = self._backButton.get_label()
 		self._backTapHandler = gtk_toolbox.TapOrHold(self._backButton)
@@ -493,12 +541,16 @@ class Dialpad(object):
 		self._backTapHandler.on_cancel = self._reset_back_button
 
 		self._window = gtk_toolbox.find_parent_window(self._numberdisplay)
+		self._keyPressEventId = 0
 
 	def enable(self):
 		self._dialButton.grab_focus()
 		self._backTapHandler.enable()
+		self._keyPressEventId = self._window.connect("key-press-event", self._on_key_press)
 
 	def disable(self):
+		self._window.disconnect(self._keyPressEventId)
+		self._keyPressEventId = 0
 		self._reset_back_button()
 		self._backTapHandler.disable()
 
@@ -538,12 +590,21 @@ class Dialpad(object):
 		"""
 		pass
 
+	def _on_key_press(self, widget, event):
+		try:
+			if event.keyval == ord("v") and event.get_state() & gtk.gdk.CONTROL_MASK:
+				contents = self._clipboard.wait_for_text()
+				if contents is not None:
+					self.set_number(contents)
+		except Exception, e:
+			self._errorDisplay.push_exception()
+
 	def _on_sms_clicked(self, widget):
 		try:
 			action = PhoneTypeSelector.ACTION_SEND_SMS
 			phoneNumber = self.get_number()
 
-			message = self._smsDialog.run(phoneNumber, "", self._window)
+			message = self._smsDialog.run(phoneNumber, (), self._window)
 			if not message:
 				phoneNumber = ""
 				action = PhoneTypeSelector.ACTION_CANCEL
@@ -563,12 +624,6 @@ class Dialpad(object):
 		except Exception, e:
 			self._errorDisplay.push_exception()
 
-	def _on_clear_number(self, *args):
-		try:
-			self.clear()
-		except Exception, e:
-			self._errorDisplay.push_exception()
-
 	def _on_digit_clicked(self, widget):
 		try:
 			self.set_number(self._phonenumber + widget.get_name()[-1])
@@ -651,7 +706,7 @@ class AccountInfo(object):
 			self._smsCheckbox.set_active(self._notifyOnSms)
 
 			self._onNotifyToggled = self._notifyCheckbox.connect("toggled", self._on_notify_toggled)
-			self._onMinutesChanged = self._minutesEntryButton.connect("clicked", self._on_minutes_changed)
+			self._onMinutesChanged = self._minutesEntryButton.connect("clicked", self._on_minutes_clicked)
 			self._onMissedToggled = self._missedCheckbox.connect("toggled", self._on_missed_toggled)
 			self._onVoicemailToggled = self._voicemailCheckbox.connect("toggled", self._on_voicemail_toggled)
 			self._onSmsToggled = self._smsCheckbox.connect("toggled", self._on_sms_toggled)
@@ -754,7 +809,7 @@ class AccountInfo(object):
 
 	def _set_callback_number(self, number):
 		try:
-			if not self._backend.is_valid_syntax(number):
+			if not self._backend.is_valid_syntax(number) and 0 < len(number):
 				self._errorDisplay.push_message("%s is not a valid callback number" % number)
 			elif number == self._backend.get_callback_number():
 				logging.warning(
@@ -802,11 +857,33 @@ class AccountInfo(object):
 		except Exception, e:
 			self._errorDisplay.push_exception()
 
-	def _on_minutes_changed(self, *args):
+	def _on_minutes_clicked(self, *args):
+		recurrenceChoices = [
+			(1, "1 minute"),
+			(3, "3 minutes"),
+			(5, "5 minutes"),
+			(10, "10 minutes"),
+			(15, "15 minutes"),
+			(30, "30 minutes"),
+			(45, "45 minutes"),
+			(60, "1 hour"),
+			(12*60, "12 hours"),
+		]
 		try:
-			recurrence = hildonize.request_number(
-				self._window, "Minutes", (1, 50), self._alarmHandler.recurrence
+			actualSelection = self._alarmHandler.recurrence
+
+			closestSelectionIndex = 0
+			for i, possible in enumerate(recurrenceChoices):
+				if possible[0] <= actualSelection:
+					closestSelectionIndex = i
+			recurrenceIndex = hildonize.touch_selector(
+				self._window,
+				"Minutes",
+				(("%s" % m[1]) for m in recurrenceChoices),
+				closestSelectionIndex,
 			)
+			recurrence = recurrenceChoices[recurrenceIndex][0]
+
 			self._update_alarm_settings(recurrence)
 		except Exception, e:
 			self._errorDisplay.push_exception()
@@ -954,6 +1031,8 @@ class RecentCallsView(object):
 		pass
 
 	def _idly_populate_recentview(self):
+		with gtk_toolbox.gtk_lock():
+			banner = hildonize.show_busy_banner_start(self._window, "Loading Recent History")
 		try:
 			self._recentmodel.clear()
 			self._isPopulated = True
@@ -976,6 +1055,9 @@ class RecentCallsView(object):
 					self._recentmodel.append(item)
 		except Exception, e:
 			self._errorDisplay.push_exception_with_lock()
+		finally:
+			with gtk_toolbox.gtk_lock():
+				hildonize.show_busy_banner_end(banner)
 
 		return False
 
@@ -992,7 +1074,7 @@ class RecentCallsView(object):
 
 			action, phoneNumber, message = self._phoneTypeSelector.run(
 				contactPhoneNumbers,
-				message = description,
+				messages = (description, ),
 				parent = self._window,
 			)
 			if action == PhoneTypeSelector.ACTION_CANCEL:
@@ -1011,6 +1093,7 @@ class MessagesView(object):
 	DATE_IDX = 1
 	HEADER_IDX = 2
 	MESSAGE_IDX = 3
+	MESSAGES_IDX = 4
 
 	def __init__(self, widgetTree, backend, errorDisplay):
 		self._errorDisplay = errorDisplay
@@ -1022,6 +1105,7 @@ class MessagesView(object):
 			gobject.TYPE_STRING, # date
 			gobject.TYPE_STRING, # header
 			gobject.TYPE_STRING, # message
+			object, # messages
 		)
 		self._messageview = widgetTree.get_widget("messages_view")
 		self._messageviewselection = None
@@ -1093,6 +1177,8 @@ class MessagesView(object):
 		pass
 
 	def _idly_populate_messageview(self):
+		with gtk_toolbox.gtk_lock():
+			banner = hildonize.show_busy_banner_start(self._window, "Loading Messages")
 		try:
 			self._messagemodel.clear()
 			self._isPopulated = True
@@ -1104,16 +1190,24 @@ class MessagesView(object):
 				self._isPopulated = False
 				messageItems = []
 
-			for header, number, relativeDate, message in messageItems:
+			for header, number, relativeDate, messages in messageItems:
 				prettyNumber = number[2:] if number.startswith("+1") else number
 				prettyNumber = make_pretty(prettyNumber)
-				message = "<b>%s - %s</b> <i>(%s)</i>\n\n%s" % (header, prettyNumber, relativeDate, message)
+
+				firstMessage = "<b>%s - %s</b> <i>(%s)</i>" % (header, prettyNumber, relativeDate)
+				newMessages = [firstMessage]
+				newMessages.extend(messages)
+
 				number = make_ugly(number)
-				row = (number, relativeDate, header, message)
+
+				row = (number, relativeDate, header, "\n".join(newMessages), newMessages)
 				with gtk_toolbox.gtk_lock():
 					self._messagemodel.append(row)
 		except Exception, e:
 			self._errorDisplay.push_exception_with_lock()
+		finally:
+			with gtk_toolbox.gtk_lock():
+				hildonize.show_busy_banner_end(banner)
 
 		return False
 
@@ -1124,11 +1218,11 @@ class MessagesView(object):
 				return
 
 			contactPhoneNumbers = [("Phone", self._messagemodel.get_value(itr, self.NUMBER_IDX))]
-			description = self._messagemodel.get_value(itr, self.MESSAGE_IDX)
+			description = self._messagemodel.get_value(itr, self.MESSAGES_IDX)
 
 			action, phoneNumber, message = self._phoneTypeSelector.run(
 				contactPhoneNumbers,
-				message = description,
+				messages = description,
 				parent = self._window,
 			)
 			if action == PhoneTypeSelector.ACTION_CANCEL:
@@ -1151,8 +1245,8 @@ class ContactsView(object):
 		self._selectedComboIndex = 0
 		self._addressBookFactories = [null_backend.NullAddressBook()]
 
-		self._booksList = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING)
-		self._booksSelectionBox = widgetTree.get_widget("addressbook_combo")
+		self._booksList = []
+		self._bookSelectionButton = widgetTree.get_widget("addressbookSelectButton")
 
 		self._isPopulated = False
 		self._contactsmodel = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING)
@@ -1177,7 +1271,7 @@ class ContactsView(object):
 		self._contactColumn.set_visible(True)
 
 		self._onContactsviewRowActivatedId = 0
-		self._onAddressbookComboChangedId = 0
+		self._onAddressbookButtonChangedId = 0
 		self._window = gtk_toolbox.find_parent_window(self._contactsview)
 		self._phoneTypeSelector = PhoneTypeSelector(widgetTree, self._backend)
 
@@ -1196,7 +1290,7 @@ class ContactsView(object):
 		self._contactsviewselection = self._contactsview.get_selection()
 		self._contactsviewselection.set_mode(gtk.SELECTION_SINGLE)
 
-		self._booksList.clear()
+		del self._booksList[:]
 		for (factoryId, bookId), (factoryName, bookName) in self.get_addressbooks():
 			if factoryName and bookName:
 				entryName = "%s: %s" % (factoryName, bookName)
@@ -1209,26 +1303,24 @@ class ContactsView(object):
 			row = (str(factoryId), bookId, entryName)
 			self._booksList.append(row)
 
-		self._booksSelectionBox.set_model(self._booksList)
-		cell = gtk.CellRendererText()
-		self._booksSelectionBox.pack_start(cell, True)
-		self._booksSelectionBox.add_attribute(cell, 'text', 2)
-
 		self._onContactsviewRowActivatedId = self._contactsview.connect("row-activated", self._on_contactsview_row_activated)
-		self._onAddressbookComboChangedId = self._booksSelectionBox.connect("changed", self._on_addressbook_combo_changed)
+		self._onAddressbookButtonChangedId = self._bookSelectionButton.connect("clicked", self._on_addressbook_button_changed)
 
 		if len(self._booksList) <= self._selectedComboIndex:
 			self._selectedComboIndex = 0
-		self._booksSelectionBox.set_active(self._selectedComboIndex)
+		self._bookSelectionButton.set_label(self._booksList[self._selectedComboIndex][2])
+
+		selectedFactoryId = self._booksList[self._selectedComboIndex][0]
+		selectedBookId = self._booksList[self._selectedComboIndex][1]
+		self.open_addressbook(selectedFactoryId, selectedBookId)
 
 	def disable(self):
 		self._contactsview.disconnect(self._onContactsviewRowActivatedId)
-		self._booksSelectionBox.disconnect(self._onAddressbookComboChangedId)
+		self._bookSelectionButton.disconnect(self._onAddressbookButtonChangedId)
 
 		self.clear()
 
-		self._booksSelectionBox.clear()
-		self._booksSelectionBox.set_model(None)
+		self._bookSelectionButton.set_label("")
 		self._contactsview.set_model(None)
 		self._contactsview.remove_column(self._contactColumn)
 
@@ -1288,6 +1380,8 @@ class ContactsView(object):
 		config.set(sectionName, "selectedAddressbook", str(self._selectedComboIndex))
 
 	def _idly_populate_contactsview(self):
+		with gtk_toolbox.gtk_lock():
+			banner = hildonize.show_busy_banner_start(self._window, "Loading Contacts")
 		try:
 			addressBook = None
 			while addressBook is not self._addressBook:
@@ -1312,17 +1406,28 @@ class ContactsView(object):
 			self._isPopulated = True
 		except Exception, e:
 			self._errorDisplay.push_exception_with_lock()
+		finally:
+			with gtk_toolbox.gtk_lock():
+				hildonize.show_busy_banner_end(banner)
 		return False
 
-	def _on_addressbook_combo_changed(self, *args, **kwds):
+	def _on_addressbook_button_changed(self, *args, **kwds):
 		try:
-			itr = self._booksSelectionBox.get_active_iter()
-			if itr is None:
+			try:
+				newSelectedComboIndex = hildonize.touch_selector(
+					self._window,
+					"Addressbook",
+					(("%s" % m[2]) for m in self._booksList),
+					self._selectedComboIndex,
+				)
+			except RuntimeError:
 				return
-			self._selectedComboIndex = self._booksSelectionBox.get_active()
-			selectedFactoryId = self._booksList.get_value(itr, 0)
-			selectedBookId = self._booksList.get_value(itr, 1)
+
+			selectedFactoryId = self._booksList[newSelectedComboIndex][0]
+			selectedBookId = self._booksList[newSelectedComboIndex][1]
 			self.open_addressbook(selectedFactoryId, selectedBookId)
+			self._selectedComboIndex = newSelectedComboIndex
+			self._bookSelectionButton.set_label(self._booksList[self._selectedComboIndex][2])
 		except Exception, e:
 			self._errorDisplay.push_exception()
 
@@ -1346,7 +1451,7 @@ class ContactsView(object):
 
 			action, phoneNumber, message = self._phoneTypeSelector.run(
 				contactPhoneNumbers,
-				message = contactName,
+				messages = (contactName, ),
 				parent = self._window,
 			)
 			if action == PhoneTypeSelector.ACTION_CANCEL:
